package org.ow2.petals.registry.database;

import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

public class NamingEnumerationImpl<T> implements NamingEnumeration<T> {
    
    private final List<T> enumeration;
    private boolean closed;

    public NamingEnumerationImpl(final List<T> list) {
        this.enumeration = new LinkedList<T>(list);
        this.closed = false;
    }
    
    /* (non-Javadoc)
     * @see javax.naming.NamingEnumeration#close()
     */
    public void close() throws NamingException {
        // TODO Auto-generated method stub
        this.enumeration.clear();
        this.closed = true;
    }

    /* (non-Javadoc)
     * @see javax.naming.NamingEnumeration#hasMore()
     */
    public boolean hasMore() throws NamingException {
        
        if (!closed) {
            synchronized (this.enumeration) {
                return this.enumeration.size() > 0;
            }
        } else {
            throw new NamingException("This naming enumeration is closed.");
        }
    }

    /* (non-Javadoc)
     * @see javax.naming.NamingEnumeration#next()
     */
    public T next() throws NamingException {
        if (!closed) {
            final T object;
            synchronized (this.enumeration) {
                if (this.enumeration.size() > 0) { 
                    object = this.enumeration.get(0);
                    this.enumeration.remove(0);
                } else {
                    throw new NoSuchElementException("No more element"); 
                }
            }
            return object;
        } else {
            throw new NamingException("This naming enumeration is closed.");
        }
    }

    /* (non-Javadoc)
     * @see java.util.Enumeration#hasMoreElements()
     */
    public boolean hasMoreElements() {
        try {
            return this.hasMore();
        } catch (NamingException e) {
            // NOP
            return false;
        }
    }

    /* (non-Javadoc)
     * @see java.util.Enumeration#nextElement()
     */
    public T nextElement() {
        try {
            return this.next();
        } catch (NamingException e) {
            // NOP
            return null;
        }
    }
}
